library spacing;

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';

class SpacingData {
  final double verticalSpacing;
  final double horizontalSpacing;
  final EdgeInsets padding;

  SpacingData({this.verticalSpacing, this.horizontalSpacing, this.padding});
}

class Spacing extends StatelessWidget {
  final double verticalSpacing;
  final double horizontalSpacing;
  final Widget child;
  final bool topSafeArea;
  final bool bottomSafeArea;

  final EdgeInsets padding;

  Spacing()
      : horizontalSpacing = null,
        verticalSpacing = null,
        child = null,
        padding = EdgeInsets.all(0),
        topSafeArea = false,
        bottomSafeArea = false;

  Spacing.vertical([this.verticalSpacing])
      : horizontalSpacing = 0,
        child = null,
        padding = EdgeInsets.all(0),
        topSafeArea = false,
        bottomSafeArea = false;

  Spacing.horizontal([this.horizontalSpacing])
      : verticalSpacing = 0,
        child = null,
        padding = EdgeInsets.all(0),
        topSafeArea = false,
        bottomSafeArea = false;

  Spacing.padding({this.padding, @required this.child})
      : verticalSpacing = 0,
        horizontalSpacing = 0,
        topSafeArea = false,
        bottomSafeArea = false;

  Spacing.topSafeArea({this.verticalSpacing})
      : child = null,
        padding = EdgeInsets.all(0),
        horizontalSpacing = 0,
        topSafeArea = true,
        bottomSafeArea = false;

  Spacing.bottomSafeArea({this.verticalSpacing})
      : child = null,
        padding = EdgeInsets.all(0),
        horizontalSpacing = 0,
        topSafeArea = false,
        bottomSafeArea = true;

  @override
  Widget build(BuildContext context) {
    if (child != null) {
      return Padding(
        child: child,
        padding: padding ?? DefaultSpacing.of(context),
      );
    }
    final result = SizedBox(
      height: verticalSpacing ?? DefaultSpacing.of(context).verticalSpacing,
      width: horizontalSpacing ?? DefaultSpacing.of(context).horizontalSpacing,
    );

    if (topSafeArea)
      return SafeArea(
        child: result,
        bottom: false,
        left: false,
        right: false,
      );
    if (topSafeArea)
      return SafeArea(
        child: result,
        bottom: false,
        left: false,
        right: false,
      );
    return result;
  }
}

class SliverSpacing extends StatelessWidget {
  final double verticalSpacing;
  final double horizontalSpacing;
  final Widget sliver;
  final bool topSafeArea;
  final bool bottomSafeArea;

  final EdgeInsets padding;

  SliverSpacing()
      : horizontalSpacing = null,
        verticalSpacing = null,
        sliver = null,
        padding = EdgeInsets.all(0),
        topSafeArea = false,
        bottomSafeArea = false;

  SliverSpacing.vertical([this.verticalSpacing])
      : horizontalSpacing = 0,
        sliver = null,
        padding = EdgeInsets.all(0),
        topSafeArea = false,
        bottomSafeArea = false;

  SliverSpacing.horizontal([this.horizontalSpacing])
      : verticalSpacing = 0,
        sliver = null,
        padding = EdgeInsets.all(0),
        topSafeArea = false,
        bottomSafeArea = false;

  SliverSpacing.padding({this.padding, @required this.sliver})
      : verticalSpacing = 0,
        horizontalSpacing = 0,
        topSafeArea = false,
        bottomSafeArea = false;

  SliverSpacing.topSafeArea({this.verticalSpacing})
      : sliver = null,
        padding = EdgeInsets.all(0),
        horizontalSpacing = 0,
        topSafeArea = true,
        bottomSafeArea = false;

  SliverSpacing.bottomSafeArea({this.verticalSpacing})
      : sliver = null,
        padding = EdgeInsets.all(0),
        horizontalSpacing = 0,
        topSafeArea = false,
        bottomSafeArea = true;

  @override
  Widget build(BuildContext context) {
    if (sliver != null) {
      return SliverPadding(
        sliver: sliver,
        padding: padding ?? DefaultSpacing.of(context).padding,
      );
    }
    final result = SliverToBoxAdapter(
      child: SizedBox(
        height: verticalSpacing ?? DefaultSpacing.of(context).verticalSpacing,
        width:
            horizontalSpacing ?? DefaultSpacing.of(context).horizontalSpacing,
      ),
    );
    if (topSafeArea)
      return SliverSafeArea(
        sliver: result,
        bottom: false,
        left: false,
        right: false,
      );
    if (topSafeArea)
      return SliverSafeArea(
        sliver: result,
        bottom: false,
        left: false,
        right: false,
      );
    return result;
  }
}

class DefaultSpacing extends InheritedTheme {
  final double verticalSpacing;
  final double horizontalSpacing;
  final EdgeInsets padding;

  DefaultSpacing({
    Key key,
    this.padding,
    this.verticalSpacing,
    this.horizontalSpacing,
    @required Widget child,
  }) : super(key: key, child: child);

  static DefaultSpacing of(BuildContext context) {
    return context.dependOnInheritedWidgetOfExactType<DefaultSpacing>() ??
        const DefaultSpacing.fallback();
  }

  const DefaultSpacing.fallback()
      : verticalSpacing = 0,
        horizontalSpacing = 0,
        padding = const EdgeInsets.all(0);

  @override
  bool updateShouldNotify(DefaultSpacing oldWidget) {
    return verticalSpacing != oldWidget.verticalSpacing ||
        horizontalSpacing != oldWidget.horizontalSpacing;
  }

  @override
  Widget wrap(BuildContext context, Widget child) {
    final DefaultSpacing defaultSpacing =
        context.findAncestorWidgetOfExactType<DefaultSpacing>();
    return identical(this, defaultSpacing)
        ? child
        : DefaultSpacing(
            verticalSpacing: verticalSpacing,
            horizontalSpacing: horizontalSpacing,
            padding: padding,
            child: child,
          );
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(
        DoubleProperty('verticalSpacing', verticalSpacing, defaultValue: null));
    properties.add(DoubleProperty('horizontalSpacing', horizontalSpacing,
        defaultValue: null));
    properties.add(
        DoubleProperty('verticalSpacing', verticalSpacing, defaultValue: null));
  }
}
